@@ -1,41 +1,44 @@
 import json
+import os
 import platform
 import shlex
+import signal
 import subprocess
 import time
 import traceback
 from argparse import Namespace
 from logging import getLogger
-from signal import signal, SIGINT, SIGTERM, strsignal
 from typing import Optional
 
+import shiboken6
+from legendary.models.game import SaveGameStatus
+
 # from PySide6.import sip
 from PySide6.QtCore import (
+    QMetaMethod,
     QObject,
     QProcess,
-    Signal,
-    QUrl,
     QRunnable,
-    QThreadPool,
-    QSettings,
     Qt,
-    Slot,
+    QThreadPool,
     QTimer,
-    QMetaMethod,
+    QUrl,
+    Signal,
+    Slot,
 )
 from PySide6.QtGui import QDesktopServices
 from PySide6.QtNetwork import QLocalServer, QLocalSocket
 from PySide6.QtWidgets import QApplication
-from legendary.models.game import SaveGameStatus
 
 from rare.lgndr.core import LegendaryCore
 from rare.models.base_game import RareGameSlim
-from rare.models.launcher import ErrorModel, Actions, FinishedModel, BaseModel, StateChangedModel
-from rare.models.options import options
+from rare.models.launcher import Actions, BaseModel, ErrorModel, FinishedModel, StateChangedModel
+from rare.utils.paths import get_rare_executable
 from rare.widgets.rare_app import RareApp, RareAppException
+
 from .cloud_sync_dialog import CloudSyncDialog, CloudSyncDialogResult
 from .console_dialog import ConsoleDialog
-from .lgd_helper import get_launch_args, InitArgs, get_configured_process, LaunchArgs, GameArgsError
+from .lgd_helper import InitParams, LaunchParams, dict_to_qprocenv, get_configured_qprocess, get_launch_params
 
 DETACHED_APP_NAMES = {
     "0a2d9f6403244d12969e11da6713137b",  # Fall Guys
@@ -43,19 +46,19 @@ DETACHED_APP_NAMES = {
     "afdb5a85efcc45d8ae8e406e2121d81c",  # Fortnite Battle Royale
     "09e442f830a341f698b4da42abd98c9b",  # Fortnite Festival
     "d8f7763e07d74c209d760a679f9ed6ac",  # Lego Fortnite
-    "Fortnite_Studio",                   # Unreal Editor for Fortnite
+    "Fortnite_Studio",  # Unreal Editor for Fortnite
     "dcfccf8d965a4f2281dddf9fead042de",  # Homeworld Remastered Collection (issue#376)
 }
 
 
 class PreLaunch(QRunnable):
     class Signals(QObject):
-        ready_to_launch = Signal(LaunchArgs)
+        ready_to_launch = Signal(LaunchParams)
         pre_launch_command_started = Signal()
         pre_launch_command_finished = Signal(int)  # exit_code
         error_occurred = Signal(str)
 
-    def __init__(self, args: InitArgs, rgame: RareGameSlim, sync_action=None):
+    def __init__(self, args: InitParams, rgame: RareGameSlim, sync_action=None):
         super(PreLaunch, self).__init__()
         self.signals = self.Signals()
         self.logger = getLogger(type(self).__name__)
@@ -77,30 +80,30 @@ class PreLaunch(QRunnable):
         else:
             return
 
-    def prepare_launch(self, args: InitArgs) -> Optional[LaunchArgs]:
+    def prepare_launch(self, args: InitParams) -> Optional[LaunchParams]:
         try:
-            launch_args = get_launch_args(self.rgame, args)
+            launch = get_launch_params(self.rgame, args)
         except Exception as e:
             self.signals.error_occurred.emit(str(e))
             return None
-        if not launch_args:
+        if not launch:
             return None
 
-        if launch_args.pre_launch_command:
-            proc = get_configured_process()
-            proc.setProcessEnvironment(launch_args.environment)
+        if launch.pre_launch_command:
+            proc = get_configured_qprocess(shlex.split(launch.pre_launch_command), launch.environment)
+            proc.setProcessChannelMode(QProcess.ProcessChannelMode.MergedChannels)
+            proc.readyReadStandardOutput.connect(
+                lambda: self.logger.info(str(proc.readAllStandardOutput().data(), "utf-8", "ignore"))
+            )
             self.signals.pre_launch_command_started.emit()
-            prelaunch = shlex.split(launch_args.pre_launch_command)
-            command = prelaunch.pop(0) if len(prelaunch) else ""
-            arguments = prelaunch if len(prelaunch) else []
-            self.logger.info("Running pre-launch command %s, %s", command, shlex.join(arguments))
-            if launch_args.pre_launch_wait:
-                proc.start(command, arguments)
+            self.logger.info("Running pre-launch command %s, %s", proc.program(), proc.arguments())
+            if launch.pre_launch_wait:
+                proc.start()
                 self.logger.info("Waiting for pre-launch command to finish")
                 proc.waitForFinished(-1)
             else:
-                proc.startDetached(command, arguments)
-        return launch_args
+                proc.startDetached()
+        return launch
 
 
 class SyncCheckWorker(QRunnable):
@@ -131,11 +134,13 @@ class RareLauncherException(RareAppException):
 
     def _handler(self, exc_type, exc_value, exc_tb) -> bool:
         try:
-            self.__app.send_message(ErrorModel(
-                app_name=self.__args.app_name,
-                action=Actions.error,
-                error_string="".join(traceback.format_exception(exc_type, exc_value, exc_tb))
-            ))
+            self.__app.send_message(
+                ErrorModel(
+                    app_name=self.__args.app_name,
+                    action=Actions.error,
+                    error_string="".join(traceback.format_exception(exc_type, exc_value, exc_tb)),
+                )
+            )
         except RuntimeError:
             pass
         return False
@@ -144,7 +149,7 @@ class RareLauncherException(RareAppException):
 class RareLauncher(RareApp):
     exit_app = Signal()
 
-    def __init__(self, args: InitArgs):
+    def __init__(self, args: InitParams):
         super(RareLauncher, self).__init__(args, f"{type(self).__name__}_{args.app_name}_{{0}}.log")
         self.socket: Optional[QLocalSocket] = None
         self.console: Optional[ConsoleDialog] = None
@@ -162,17 +167,21 @@ class RareLauncher(RareApp):
         if not game:
             self.logger.error(f"Game {args.app_name} not found. Exiting")
             return
-        self.rgame = RareGameSlim(self.core, game, self)
+        self.rgame = RareGameSlim(self.settings, self.core, game)
 
-        language = self.settings.value(*options.language)
-        self.load_translator(language)
-
-        # if QSettings(self).value(*options.log_games) or (
-        #     game.app_name in DETACHED_APP_NAMES and platform.system() == "Windows"
-        # ):
-        if QSettings(self).value(*options.log_games):
+        if args.show_console:
             self.console = ConsoleDialog(game.app_title)
             self.console.show()
+            self.game_process.stateChanged.connect(
+                lambda s: self.console.kill_button.setEnabled(
+                    self.game_process.state() == QProcess.ProcessState.Running
+                )
+            )
+            self.game_process.stateChanged.connect(
+                lambda s: self.console.terminate_button.setEnabled(
+                    self.game_process.state() == QProcess.ProcessState.Running
+                )
+            )
 
         self.sync_dialog: Optional[CloudSyncDialog] = None
 
@@ -201,23 +210,25 @@ class RareLauncher(RareApp):
 
     @Slot()
     def __proc_log_stdout(self):
-        self.console.log_stdout(
-            self.game_process.readAllStandardOutput().data().decode("utf-8", "ignore")
-        )
+        self.console.log_stdout(self.game_process.readAllStandardOutput().data().decode("utf-8", "ignore"))
 
     @Slot()
     def __proc_log_stderr(self):
-        self.console.log_stderr(
-            self.game_process.readAllStandardError().data().decode("utf-8", "ignore")
-        )
+        self.console.log_stderr(self.game_process.readAllStandardError().data().decode("utf-8", "ignore"))
 
     @Slot()
     def __proc_term(self):
-        self.game_process.terminate()
+        if platform.system() == "Windows":
+            self.game_process.terminate()
+        else:
+            os.kill(self.game_process.processId(), signal.SIGINT)
 
     @Slot()
     def __proc_kill(self):
-        self.game_process.kill()
+        if platform.system() == "Windows":
+            self.game_process.kill()
+        else:
+            os.kill(self.game_process.processId(), signal.SIGINT)
 
     def new_server_connection(self):
         if self.socket is not None:
@@ -289,75 +300,112 @@ class RareLauncher(RareApp):
         if self.console:
             self.console.on_process_exit(self.core.get_game(self.rgame.app_name).app_title, exit_code)
 
-        self.send_message(FinishedModel(
-            action=Actions.finished,
-            app_name=self.rgame.app_name,
-            exit_code=exit_code,
-            playtime=int(time.time() - self.start_time),
-        ))
+        self.send_message(
+            FinishedModel(
+                action=Actions.finished,
+                app_name=self.rgame.app_name,
+                exit_code=exit_code,
+                playtime=int(time.time() - self.start_time),
+            )
+        )
         self.stop()
 
     @Slot(object)
-    def launch_game(self, args: LaunchArgs):
+    def launch_game(self, params: LaunchParams):
         # should never happen
-        if not args:
+        if not params:
             self.stop()
             return
         if self.console:
-            self.console.set_env(args.environment)
+            self.console.set_env(params.environment)
         self.start_time = time.time()
 
         if self.args.dry_run:
             self.logger.info("Dry run %s (%s)", self.rgame.app_title, self.rgame.app_name)
-            self.logger.info("%s %s", args.executable, " ".join(args.arguments))
+            self.logger.info("Command: %s, %s", params.executable, " ".join(params.arguments))
             if self.console:
                 self.console.log(f"Dry run {self.rgame.app_title} ({self.rgame.app_name})")
-                self.console.log(f"{shlex.join((args.executable, *args.arguments))}")
+                self.console.log(f"{shlex.join((params.executable, *params.arguments))}")
                 self.console.accept_close = True
             self.stop()
             return
 
-        if args.is_origin_game:
+        if platform.system() == "Windows" and params.is_origin_game:
             # executable is a protocol link (link2ea://launchgame/...)
-            QDesktopServices.openUrl(QUrl(args.executable))
+            QDesktopServices.openUrl(QUrl(params.executable))
             self.stop()  # stop because it is not a subprocess
             return
 
-        self.logger.debug("Launch command %s, %s", args.executable, " ".join(args.arguments))
-        self.logger.debug("Working directory %s", args.working_directory)
+        self.logger.info("Starting %s (%s)", self.rgame.app_title, self.rgame.app_name)
+        self.logger.info("Command: %s, %s", params.executable, " ".join(params.arguments))
+        self.logger.debug("Working directory %s", params.working_directory)
 
         if self.rgame.app_name in DETACHED_APP_NAMES and platform.system() == "Windows":
             if self.console:
-                self.console.log(f"Launching {args.executable} as a detached process")
+                self.console.log(f"Launching {params.executable} as a detached process")
             subprocess.Popen(
-                (args.executable, *args.arguments),
-                cwd=args.working_directory,
-                env={i: args.environment.value(i) for i in args.environment.keys()},
+                (params.executable, *params.arguments),
+                stdin=None,
+                stdout=None,
+                stderr=None,
+                cwd=params.working_directory,
+                env=params.environment,
                 shell=True,
                 creationflags=subprocess.DETACHED_PROCESS,
             )
             self.stop()  # stop because we do not attach to the output
             return
 
-        if args.working_directory:
-            self.game_process.setWorkingDirectory(args.working_directory)
-        self.game_process.setProcessEnvironment(args.environment)
+        if platform.system() in {"Linux", "FreeBSD"}:
+            cmd_line = get_rare_executable()
+            executable, arguments = cmd_line[0], cmd_line[1:]
+
+            if appid := os.environ.get("SteamGameId", False):
+                params.environment["SteamGameId"] = appid
+            elif params.environment.get("SteamGameId", False):
+                appid = params.environment["SteamGameId"]
+
+            self.game_process.setProgram(executable)
+            # TODO: Add "SteamLauch" and "AppId=xxxxxx" here for steamdeck/gamescope
+            try:
+                appid = int(appid) >> 32
+            except ValueError:
+                pass
+            self.game_process.setArguments(
+                [*arguments, "subreaper", "SteamLaunch", f"AppId={appid}", "--", params.executable, *params.arguments]
+            )
+            self.game_process.setUnixProcessParameters(
+                QProcess.UnixProcessFlag.ResetSignalHandlers | QProcess.UnixProcessFlag.CreateNewSession
+            )
+        else:
+            self.game_process.setProgram(params.executable)
+            self.game_process.setArguments(params.arguments)
+
+        if params.working_directory:
+            self.game_process.setWorkingDirectory(params.working_directory)
+
+        if self.args.debug and self.console:
+            self.console.log(str(self.game_process.program()))
+            self.console.log(shlex.join(self.game_process.arguments()))
+
+        self.game_process.setProcessEnvironment(dict_to_qprocenv(params.environment))
         # send start message after process started
-        self.game_process.started.connect(lambda: self.send_message(StateChangedModel(
-            action=Actions.state_update, app_name=self.rgame.app_name,
-            new_state=StateChangedModel.States.started
-        )))
-        # self.logger.debug("Executing prelaunch command %s, %s", args.executable, args.arguments)
-        self.game_process.start(args.executable, args.arguments)
+        self.game_process.started.connect(
+            lambda: self.send_message(
+                StateChangedModel(
+                    action=Actions.state_update,
+                    app_name=self.rgame.app_name,
+                    new_state=StateChangedModel.States.started,
+                )
+            )
+        )
+        self.game_process.start()
 
     def error_occurred(self, error_str: str):
         self.logger.warning(error_str)
         if self.console:
             self.console.on_process_exit(self.core.get_game(self.rgame.app_name).app_title, error_str)
-        self.send_message(ErrorModel(
-            error_string=error_str, app_name=self.rgame.app_name,
-            action=Actions.error
-        ))
+        self.send_message(ErrorModel(error_string=error_str, app_name=self.rgame.app_name, action=Actions.error))
         self.stop()
 
     def start_prepare(self, sync_action=None):
@@ -415,7 +463,7 @@ class RareLauncher(RareApp):
         else:
             self.start_prepare()
 
-    def stop(self):
+    def stop(self, sig: int = signal.SIGINT):
         try:
             if self.console:
                 self.game_process.readyReadStandardOutput.disconnect()
@@ -427,10 +475,17 @@ class RareLauncher(RareApp):
         except (TypeError, RuntimeError) as e:
             self.logger.error("Failed to disconnect process signals: %s", e)
 
-        if self.game_process.state() != QProcess.ProcessState.NotRunning:
-            self.game_process.kill()
-        exit_code = self.game_process.exitCode()
-        self.game_process.deleteLater()
+        if shiboken6.isValid(self.game_process):  # pylint: disable=E1101
+            if self.game_process.state() != QProcess.ProcessState.NotRunning:
+                if sig == signal.SIGTERM:
+                    self.__proc_term()
+                elif sig == signal.SIGINT:
+                    self.__proc_kill()
+            self.game_process.waitForFinished()
+            exit_code = self.game_process.exitCode()
+            self.game_process.deleteLater()
+        else:
+            exit_code = 0
 
         self.logger.info("Stopping server %s", self.server.socketDescriptor())
         try:
@@ -446,8 +501,8 @@ class RareLauncher(RareApp):
             self.console.on_process_exit(self.rgame.app_title, exit_code)
 
 
-def launch(args: Namespace) -> int:
-    args = InitArgs.from_argparse(args)
+def launcher(args: Namespace) -> int:
+    args = InitParams.from_argparse(args)
 
     QApplication.setAttribute(Qt.ApplicationAttribute.AA_EnableHighDpiScaling, True)
     QApplication.setAttribute(Qt.ApplicationAttribute.AA_UseHighDpiPixmaps, True)
@@ -457,16 +512,17 @@ def launch(args: Namespace) -> int:
 
     # This prevents ghost QLocalSockets, which block the name, which makes it unable to start
     # No handling for SIGKILL
-    def sighandler(s, frame):
-        app.logger.info("%s received. Stopping", strsignal(s))
-        app.stop()
+    def signal_handler(sig, frame):
+        app.logger.info("%s received. Stopping", signal.strsignal(sig))
+        app.stop(sig)
         app.exit(1)
+        return 1
 
-    signal(SIGINT, sighandler)
-    signal(SIGTERM, sighandler)
+    signal.signal(signal.SIGTERM, signal_handler)
+    signal.signal(signal.SIGINT, signal_handler)
 
     if not app.success:
-        app.stop()
+        app.stop(signal.SIGINT)
         app.exit(1)
         return 1
 
@@ -480,3 +536,6 @@ def launch(args: Namespace) -> int:
         # if not sip.isdeleted(app.server):
         #     app.server.close()
     return exit_code
+
+
+__all__ = ["launcher"]
