@@ -1,8 +1,9 @@
 import sys
 
 from PySide6.QtCore import QSize
-from PySide6.QtWidgets import QDialog, QApplication, QVBoxLayout
-from legendary.core import LegendaryCore
+from PySide6.QtWidgets import QApplication, QDialog, QVBoxLayout
+
+from rare.lgndr.core import LegendaryCore
 
 from . import StoreTab
 
@@ -22,10 +23,13 @@ class StoreWindow(QDialog):
 
 
 if __name__ == "__main__":
-    import rare.resources.static_css
+    import logging
+
     # import rare.resources.stylesheets.RareStyle
     from rare.utils.misc import set_style_sheet
 
+    logging.getLogger().setLevel(logging.DEBUG)
+
     app = QApplication(sys.argv)
     app.setApplicationName("Rare")
     app.setOrganizationName("Rare")
