@@ -4,23 +4,26 @@ import platform
 from dataclasses import dataclass, field
 from datetime import datetime, timezone
 from logging import getLogger
+from secrets import token_hex
 from threading import Lock
-from typing import List, Optional, Dict, Set
+from typing import Dict, List, Optional, Set, Tuple
 
-from PySide6.QtCore import QRunnable, Slot, QProcess, QThreadPool
-from PySide6.QtGui import QPixmap
 from legendary.lfs import eos
 from legendary.models.game import Game, InstalledGame
+from PySide6.QtCore import QProcess, QRunnable, QThreadPool, Slot
+from PySide6.QtGui import QPixmap
 
-from rare.models.image import ImageSize
 from rare.lgndr.core import LegendaryCore
 from rare.models.base_game import RareGameBase, RareGameSlim
+from rare.models.image import ImageSize
 from rare.models.install import InstallOptionsModel, UninstallOptionsModel
+from rare.models.settings import RareAppSettings, app_settings
 from rare.shared.game_process import GameProcess
 from rare.shared.image_manager import ImageManager
-from rare.utils.paths import data_dir, get_rare_executable
+from rare.utils import config_helper as config
+from rare.utils.paths import compat_shaders_dir, data_dir, get_rare_executable
 from rare.utils.steam_grades import get_rating
-from rare.utils.config_helper import set_envvar, get_boolean
+from rare.utils.workarounds import apply_workarounds
 
 logger = getLogger("RareGame")
 
@@ -32,11 +35,11 @@ class RareGame(RareGameSlim):
         queue_pos: Optional[int] = None
         last_played: datetime = datetime.min.replace(tzinfo=timezone.utc)
         grant_date: datetime = datetime.min.replace(tzinfo=timezone.utc)
-        steam_appid: Optional[int] = None
+        steam_appid: Optional[str] = None
         steam_grade: Optional[str] = None
         steam_date: datetime = datetime.min.replace(tzinfo=timezone.utc)
         steam_shortcut: Optional[int] = None
-        tags: List[str] = field(default_factory=list)
+        tags: Tuple[str, ...] = field(default_factory=tuple)
 
         # For compatibility with previously created game metadata
         @staticmethod
@@ -51,11 +54,11 @@ class RareGame(RareGameSlim):
                 queue_pos=data.get("queue_pos", None),
                 last_played=RareGame.Metadata.parse_date(data.get("last_played", "")),
                 grant_date=RareGame.Metadata.parse_date(data.get("grant_date", "")),
-                steam_appid=data.get("steam_appid", None),
+                steam_appid=str(appid) if (appid := data.get("steam_appid", "")) else None,
                 steam_grade=data.get("steam_grade", None),
                 steam_date=RareGame.Metadata.parse_date(data.get("steam_date", "")),
                 steam_shortcut=data.get("steam_shortcut", None),
-                tags=data.get("tags", []),
+                tags=data.get("tags", ()),
             )
 
         @property
@@ -65,7 +68,7 @@ class RareGame(RareGameSlim):
                 queue_pos=self.queue_pos,
                 last_played=self.last_played.isoformat() if self.last_played else datetime.min.replace(tzinfo=timezone.utc),
                 grant_date=self.grant_date.isoformat() if self.grant_date else datetime.min.replace(tzinfo=timezone.utc),
-                steam_appid=self.steam_appid,
+                steam_appid=str(self.steam_appid) if self.steam_appid else None,
                 steam_grade=self.steam_grade,
                 steam_date=self.steam_date.isoformat() if self.steam_date else datetime.min.replace(tzinfo=timezone.utc),
                 steam_shortcut=self.steam_shortcut,
@@ -75,8 +78,14 @@ class RareGame(RareGameSlim):
         def __bool__(self):
             return self.queued or self.queue_pos is not None or self.last_played is not None
 
-    def __init__(self, legendary_core: LegendaryCore, image_manager: ImageManager, game: Game, parent=None):
-        super(RareGame, self).__init__(legendary_core, game, parent=parent)
+    def __init__(
+        self,
+        settings: RareAppSettings,
+        legendary_core: LegendaryCore,
+        image_manager: ImageManager,
+        game: Game,
+    ):
+        super(RareGame, self).__init__(settings, legendary_core, game)
         self.__origin_install_path: Optional[str] = None
         self.__origin_install_size: Optional[int] = None
 
@@ -100,6 +109,7 @@ class RareGame(RareGameSlim):
         self.progress: int = 0
         self.signals.progress.start.connect(lambda: self.__on_progress_update(0))
         self.signals.progress.update.connect(self.__on_progress_update)
+        self.__steam_grade_pending: bool = False
 
         self.game_process = GameProcess(self.game)
         self.game_process.launched.connect(self.__game_launched)
@@ -107,6 +117,9 @@ class RareGame(RareGameSlim):
         if self.is_installed and not self.is_dlc:
             self.game_process.connect_to_server(on_startup=True)
 
+        if self.is_installed:
+            apply_workarounds(self.app_name)
+
     def add_dlc(self, dlc) -> None:
         # lk: plug dlc progress signals to the game's
         dlc.signals.progress.start.connect(self.signals.progress.start)
@@ -117,7 +130,7 @@ class RareGame(RareGameSlim):
     def __on_progress_update(self, progress: int):
         self.progress = progress
 
-    def worker(self) -> Optional[QRunnable]:
+    def get_worker(self) -> Optional[QRunnable]:
         return self.__worker
 
     def set_worker(self, worker: Optional[QRunnable]):
@@ -147,7 +160,7 @@ class RareGame(RareGameSlim):
     __metadata_lock: Lock = Lock()
 
     @staticmethod
-    def __load_metadata_json() -> Dict:
+    def __load_metadata_json() -> Optional[Dict]:
         if RareGame.__metadata_json is None:
             metadata = {}
             file = os.path.join(data_dir(), "game_meta.json")
@@ -180,8 +193,9 @@ class RareGame(RareGameSlim):
 
     def update_game(self):
         self.game = self.core.get_game(
-            self.app_name, update_meta=False,
-            platform=self.igame.platform if self.igame else self.default_platform
+            self.app_name,
+            update_meta=False,
+            platform=self.igame.platform if self.igame else self.default_platform,
         )
 
     def update_igame(self):
@@ -274,8 +288,7 @@ class RareGame(RareGameSlim):
 
         @return bool If the game should be considered installed
         """
-        return (self.igame is not None) \
-            or (self.is_origin and self.__origin_install_path is not None)
+        return (self.igame is not None) or (self.is_origin and self.__origin_install_path is not None)
 
     def set_installed(self, installed: bool) -> None:
         """!
@@ -289,6 +302,7 @@ class RareGame(RareGameSlim):
         """
         if installed:
             self.update_igame()
+            apply_workarounds(self.app_name)
             if not self.is_dlc:
                 self.core.egstore_delete(self.igame)
                 self.core.egstore_write(self.igame.app_name)
@@ -301,7 +315,7 @@ class RareGame(RareGameSlim):
             self.core.egstore_delete(self.igame)
             self.igame = None
             self.signals.game.uninstalled.emit(self.app_name)
-        self.set_pixmap()
+        self.__update_pixmap()
 
     @property
     def can_run_offline(self) -> bool:
@@ -413,6 +427,10 @@ class RareGame(RareGameSlim):
         # Asset infos are usually None, but there was a bug, that it was an empty GameAsset class
         return not self.game.asset_infos or not next(iter(self.game.asset_infos.values())).app_name
 
+    @property
+    def is_android_only(self) -> bool:
+        return self.is_non_asset and self.is_android
+
     @property
     def is_ubisoft(self) -> bool:
         return self.game.partner_link_type == "ubisoft"
@@ -436,34 +454,44 @@ class RareGame(RareGameSlim):
             self.store_igame()
             self.signals.widget.update.emit()
 
-    def steam_grade(self) -> str:
-        if platform.system() == "Windows" or self.is_unreal:
-            return "na"
-        if self.metadata.steam_grade != "pending":
-            elapsed_time = abs(datetime.now(timezone.utc) - self.metadata.steam_date)
-            if elapsed_time.days > 3:
-                logger.info("Refreshing ProtonDB grade for %s", self.app_title)
-                worker = QRunnable.create(self.set_steam_grade)
-                QThreadPool.globalInstance().start(worker)
-                self.metadata.steam_grade = "pending"
-        return self.metadata.steam_grade
+    def reset_steam_date(self):
+        self.metadata.steam_date = datetime.min.replace(tzinfo=timezone.utc)
+        self.signals.widget.update.emit()
 
     @property
-    def steam_appid(self) -> Optional[int]:
+    def steam_appid(self) -> Optional[str]:
         return self.metadata.steam_appid
 
-    def set_steam_appid(self, appid: int):
-        set_envvar(self.app_name, "SteamAppId", str(appid))
-        set_envvar(self.app_name, "SteamGameId", str(appid))
-        set_envvar(self.app_name, "STEAM_COMPAT_APP_ID", str(appid))
+    @steam_appid.setter
+    def steam_appid(self, appid: str) -> None:
+        config.adjust_envvar(self.app_name, "SteamAppId", appid)
+        config.adjust_envvar(self.app_name, "SteamGameId", appid)
+        config.adjust_envvar(self.app_name, "STEAM_COMPAT_APP_ID", f"rare_{self.app_name}")
+        config.adjust_envvar(self.app_name, "UMU_ID", f"umu-{appid}" if appid else "umu-default")
         self.metadata.steam_appid = appid
+        self.__save_metadata()
+
+    def get_steam_grade(self) -> str:
+        if platform.system() == "Windows" or self.is_unreal:
+            return "na"
+        if self.__steam_grade_pending:
+            return "pending"
+        elapsed_time = abs(datetime.now(timezone.utc) - self.metadata.steam_date)
+        if elapsed_time.days > 3:
+            logger.info("Refreshing ProtonDB grade for %s", self.app_title)
+            worker = QRunnable.create(self.set_steam_grade)
+            self.__steam_grade_pending = True
+            QThreadPool.globalInstance().start(worker)
+            return "pending"
+        return self.metadata.steam_grade
 
     def set_steam_grade(self) -> None:
-        appid, grade = get_rating(self.core, self.app_name)
+        appid, grade = get_rating(self.core, self.app_name, self.steam_appid)
         if appid and self.steam_appid is None:
-            self.set_steam_appid(appid)
+            self.steam_appid = appid
         self.metadata.steam_grade = grade
         self.metadata.steam_date = datetime.now(timezone.utc)
+        self.__steam_grade_pending = False
         self.__save_metadata()
         self.signals.widget.update.emit()
 
@@ -471,16 +499,28 @@ class RareGame(RareGameSlim):
         if not (entitlements := self.core.lgd.entitlements):
             return self.metadata.grant_date
         if self.metadata.grant_date == datetime.min.replace(tzinfo=timezone.utc) or force:
-            logger.debug("Grant date for %s not found in metadata, resolving", self.app_name)
+            logger.info("Grant date for %s not found in metadata, resolving", self.app_name)
             matching = filter(lambda ent: ent["namespace"] == self.game.namespace, entitlements)
             entitlement = next(matching, None)
-            grant_date = datetime.fromisoformat(
-                entitlement["grantDate"].replace("Z", "+00:00")
-            ) if entitlement else datetime.min.replace(tzinfo=timezone.utc)
+            grant_date = (
+                datetime.fromisoformat(entitlement["grantDate"].replace("Z", "+00:00"))
+                if entitlement
+                else datetime.min.replace(tzinfo=timezone.utc)
+            )
             self.metadata.grant_date = grant_date
             self.__save_metadata()
         return self.metadata.grant_date
 
+    @property
+    def tags(self) -> Tuple[str, ...]:
+        return tuple(tag for tag in map(lambda x: x.lower().strip(), self.metadata.tags) if bool(tag))
+
+    @tags.setter
+    def tags(self, tags: Tuple[str, ...]) -> None:
+        self.metadata.tags = tuple(tag for tag in map(lambda x: x.lower().strip(), tags) if bool(tag))
+        logger.debug(f"Saving tags for {self.game.app_title}: {self.metadata.tags}")
+        self.__save_metadata()
+
     def set_origin_attributes(self, path: str, size: int = 0) -> None:
         self.__origin_install_path = path
         self.__origin_install_size = size
@@ -488,7 +528,7 @@ class RareGame(RareGameSlim):
             self.signals.game.installed.emit(self.app_name)
         else:
             self.signals.game.uninstalled.emit(self.app_name)
-        self.set_pixmap()
+        self.__update_pixmap()
 
     @property
     def can_launch(self) -> bool:
@@ -503,37 +543,30 @@ class RareGame(RareGameSlim):
     def get_pixmap(self, preset: ImageSize.Preset, color=True) -> QPixmap:
         return self.image_manager.get_pixmap(self.app_name, preset, color)
 
-    @Slot(object)
-    def set_pixmap(self):
-        # self.pixmap = not self.image_manager.get_pixmap(self.app_name, self.is_installed).isNull()
-        self.has_pixmap = True
+    @Slot()
+    def __update_pixmap(self):
+        self.has_pixmap = self.image_manager.has_pixmaps(self.app_name)
         if self.has_pixmap:
             self.signals.widget.update.emit()
 
     def load_pixmaps(self):
-        """ Do not call this function, call set_pixmap instead. This is only used for initial image loading """
+        """Do not call this function, call set_pixmap instead. This is only used for initial image loading"""
         if not self.has_pixmap:
-            self.image_manager.download_image(self.game, self.set_pixmap, 0, False)
+            self.image_manager.download_image(self.game, self.__update_pixmap, 0, False)
 
     def refresh_pixmap(self):
-        self.image_manager.download_image(self.game, self.set_pixmap, 0, True)
+        self.image_manager.download_image(self.game, self.__update_pixmap, 0, True)
 
     def install(self) -> bool:
         if not self.is_idle:
             return False
-        self.signals.game.install.emit(
-            InstallOptionsModel(app_name=self.app_name)
-        )
+        self.signals.game.install.emit(InstallOptionsModel(app_name=self.app_name))
         return True
 
     def modify(self) -> bool:
         if not self.is_idle:
             return False
-        self.signals.game.install.emit(
-            InstallOptionsModel(
-                app_name=self.app_name, reset_sdl=True
-            )
-        )
+        self.signals.game.install.emit(InstallOptionsModel(app_name=self.app_name, reset_sdl=True))
         return True
 
     def repair(self, repair_and_update) -> bool:
@@ -541,7 +574,10 @@ class RareGame(RareGameSlim):
             return False
         self.signals.game.install.emit(
             InstallOptionsModel(
-                app_name=self.app_name, repair_mode=True, repair_and_update=repair_and_update, update=repair_and_update
+                app_name=self.app_name,
+                repair_mode=True,
+                repair_and_update=repair_and_update,
+                update=repair_and_update,
             )
         )
         return True
@@ -549,13 +585,12 @@ class RareGame(RareGameSlim):
     def uninstall(self) -> bool:
         if not self.is_idle:
             return False
-        self.signals.game.uninstall.emit(
-            UninstallOptionsModel(app_name=self.app_name, keep_config=self.sdl_name is not None)
-        )
+        self.signals.game.uninstall.emit(UninstallOptionsModel(app_name=self.app_name, keep_config=self.sdl_name is not None))
         return True
 
     def launch(
         self,
+        debug: bool = False,
         offline: bool = False,
         skip_update_check: bool = False,
         wine_bin: Optional[str] = None,
@@ -566,25 +601,51 @@ class RareGame(RareGameSlim):
 
         cmd_line = get_rare_executable()
         executable, args = cmd_line[0], cmd_line[1:]
-        args.extend(("launch", self.app_name))
-        if offline or get_boolean(self.app_name, "offline", fallback=False):
+        args.append("launch")
+        if offline or config.get_boolean(self.app_name, "offline", fallback=False):
             args.append("--offline")
-        if skip_update_check or get_boolean(self.app_name, "skip_update_check", fallback=False):
+        if debug:
+            args.append("--debug")
+        if self.settings.get_value(app_settings.log_games):
+            args.append("--show-console")
+        if skip_update_check or config.get_boolean(self.app_name, "skip_update_check", fallback=False):
             args.append("--skip-update-check")
         if wine_bin:
             args.extend(["--wine-bin", wine_bin])
         if wine_pfx:
             args.extend(["--wine-prefix", wine_pfx])
+        args.append(self.app_name)
+
+        if not config.get_envvar(self.app_name, "STORE", False):
+            config.set_envvar(self.app_name, "STORE", "egs")
+        if not config.get_envvar(self.app_name, "UMU_ID", False):
+            config.set_envvar(
+                self.app_name,
+                "UMU_ID",
+                f"umu-{self.metadata.steam_appid}" if self.metadata.steam_appid else "umu-default",
+            )
+        if self.settings.get_with_global(app_settings.local_shader_cache, self.app_name):
+            config.set_envvar(
+                self.app_name,
+                "STEAM_COMPAT_SHADER_PATH",
+                compat_shaders_dir(self.folder_name).as_posix(),
+            )
+        else:
+            config.remove_envvar(self.app_name, "STEAM_COMPAT_SHADER_PATH")
+        config.save_config()
 
         logger.info(f"Starting game process: ({executable} {' '.join(args)})")
-        QProcess.startDetached(executable, args)
+        proc = QProcess()
+        proc.setProgram(executable)
+        proc.setArguments(args)
+        proc.startDetached()
         self.game_process.connect_to_server(on_startup=False)
         return True
 
 
 class RareEosOverlay(RareGameBase):
-    def __init__(self, legendary_core: LegendaryCore, game: Game, parent=None):
-        super(RareEosOverlay, self).__init__(legendary_core, game, parent=parent)
+    def __init__(self, legendary_core: LegendaryCore, game: Game):
+        super(RareEosOverlay, self).__init__(legendary_core, game)
         self.igame: Optional[InstalledGame] = self.core.lgd.get_overlay_install_info()
 
     @property
@@ -630,9 +691,7 @@ class RareEosOverlay(RareGameBase):
             return []
         return installs
 
-    def enable(
-        self, prefix: Optional[str] = None, path: Optional[str] = None
-    ) -> bool:
+    def enable(self, prefix: Optional[str] = None, path: Optional[str] = None) -> bool:
         if self.is_enabled(prefix):
             return False
         if not path:
@@ -676,7 +735,9 @@ class RareEosOverlay(RareGameBase):
             InstallOptionsModel(
                 app_name=self.app_name,
                 base_path=self.core.get_default_install_dir(),
-                platform="Windows", update=self.is_installed, overlay=True
+                platform="Windows",
+                update=self.is_installed,
+                overlay=True,
             )
         )
         return True
@@ -684,7 +745,5 @@ class RareEosOverlay(RareGameBase):
     def uninstall(self) -> bool:
         if not self.is_idle or not self.is_installed:
             return False
-        self.signals.game.uninstall.emit(
-            UninstallOptionsModel(app_name=self.app_name)
-        )
+        self.signals.game.uninstall.emit(UninstallOptionsModel(app_name=self.app_name))
         return True
