@@ -17,10 +17,7 @@ class BaseModel:
 
     @classmethod
     def from_json(cls, data: Dict):
-        return cls(
-            action=data["action"],
-            app_name=data["app_name"]
-        )
+        return cls(action=data["action"], app_name=data["app_name"])
 
 
 @dataclass
@@ -52,7 +49,7 @@ class StateChangedModel(BaseModel):
         return cls(
             action=data["action"],
             app_name=data["app_name"],
-            new_state=data["new_state"]
+            new_state=data["new_state"],
         )
 
 
@@ -62,7 +59,4 @@ class ErrorModel(BaseModel):
 
     @classmethod
     def from_json(cls, data: Dict):
-        return cls(
-            **vars(BaseModel.from_json(data)),
-            error_string=data["error_string"]
-        )
+        return cls(**vars(BaseModel.from_json(data)), error_string=data["error_string"])
