@@ -1,71 +1,95 @@
 import hashlib
 import json
 import pickle
+import threading
 import zlib
-
-# from concurrent import futures
 from logging import getLogger
+from multiprocessing import cpu_count
 from pathlib import Path
-from typing import TYPE_CHECKING, Optional, Set
-from typing import Tuple, Dict, Union, Type, List, Callable
+from typing import TYPE_CHECKING, Any, Callable, Dict, List, Optional, Set, Tuple, Type, Union
 
 import requests
-from PySide6.QtCore import Qt, Signal, QObject, QSize, QThreadPool, QRunnable, QRect, QRectF, Slot
-from PySide6.QtGui import QPixmap, QImage, QPainter, QPainterPath, QBrush, QTransform, QPen
-from PySide6.QtWidgets import QApplication
 from legendary.models.game import Game
+from PySide6.QtCore import (
+    QObject,
+    QRect,
+    QRectF,
+    QRunnable,
+    QSize,
+    Qt,
+    QThreadPool,
+    Signal,
+    Slot,
+)
+from PySide6.QtGui import (
+    QBrush,
+    QImage,
+    QPainter,
+    QPainterPath,
+    QPen,
+    QPixmap,
+    QTransform,
+)
 
 from rare.lgndr.core import LegendaryCore
 from rare.models.image import ImageSize, ImageType
 from rare.models.signals import GlobalSignals
 from rare.utils.paths import (
+    desktop_icon_path,
+    desktop_icon_suffix,
     image_dir,
     image_dir_game,
+    image_icon_path,
     image_tall_path,
     image_wide_path,
-    image_icon_path,
     resources_path,
-    desktop_icon_suffix,
-    desktop_icon_path,
 )
 
-# from requests_futures.sessions import FuturesSession
-
 if TYPE_CHECKING:
     pass
 
-logger = getLogger("ImageManager")
+
+class ImageWorkerSignals(QObject):
+    # object: Game
+    completed = Signal(object)
+
+
+class ImageWorker(QRunnable):
+    def __init__(self, func: Callable[[Game, bool], None], game: Game, force: bool):
+        super(ImageWorker, self).__init__()
+        self.signals = ImageWorkerSignals()
+        self.setAutoDelete(True)
+        self.func: Callable[[Game, bool], None] = func
+        self.game: Game = game
+        self.force: bool = force
+
+    def run(self):
+        self.func(self.game, self.force)
+        self.signals.completed.emit(self.game)
 
 
 class ImageManager(QObject):
-    class Worker(QRunnable):
-        class Signals(QObject):
-            # object: Game
-            completed = Signal(object)
-
-        def __init__(self, func: Callable, updates: List, json_data: Dict, game: Game):
-            super(ImageManager.Worker, self).__init__()
-            self.signals = ImageManager.Worker.Signals()
-            self.setAutoDelete(True)
-            self.func = func
-            self.updates = updates
-            self.json_data = json_data
-            self.game = game
-
-        def run(self):
-            self.func(self.updates, self.json_data, self.game)
-            logger.debug(f"Emitting singal for {self.game.app_name} ({self.game.app_title})")
-            self.signals.completed.emit(self.game)
-
     def __init__(self, signals: GlobalSignals, core: LegendaryCore):
+        self.logger = getLogger(type(self).__name__)
+        self.__cache_version = 2
         # lk: the ordering in __img_types matters for the order of fallbacks
         # {'AndroidIcon', 'DieselGameBox', 'DieselGameBoxLogo', 'DieselGameBoxTall', 'DieselGameBoxWide',
         #  'ESRB', 'Featured', 'OfferImageTall', 'OfferImageWide', 'Screenshot', 'Thumbnail'}
-        self.__img_tall_types: Tuple = ("DieselGameBoxTall", "OfferImageTall", "Thumbnail")
-        self.__img_wide_types: Tuple = ("DieselGameBoxWide", "DieselGameBox", "OfferImageWide", "Screenshot")
+        self.__img_tall_types: Tuple = (
+            "DieselGameBoxTall",
+            "OfferImageTall",
+            "Thumbnail",
+        )
+        self.__img_wide_types: Tuple = (
+            "DieselGameBoxWide",
+            "DieselGameBox",
+            "OfferImageWide",
+            "Screenshot",
+        )
         self.__img_logo_types: Tuple = ("DieselGameBoxLogo",)
         self.__img_types: Tuple = self.__img_tall_types + self.__img_wide_types + self.__img_logo_types
         self.__dl_retries = 1
+        self.__worker_lock = threading.Lock()
         self.__worker_app_names: Set[str] = set()
         super(ImageManager, self).__init__()
         self.signals = signals
@@ -74,10 +98,10 @@ class ImageManager(QObject):
         self.image_dir: Path = image_dir()
         if not self.image_dir.is_dir():
             self.image_dir.mkdir()
-            logger.info(f"Created image directory at {self.image_dir}")
+            self.logger.info("Created image directory at %s", self.image_dir)
 
-        self.threadpool = QThreadPool()
-        self.threadpool.setMaxThreadCount(6)
+        self.threadpool = QThreadPool(self)
+        self.threadpool.setMaxThreadCount(min(cpu_count() * 2, 16))
 
     @staticmethod
     def __img_json(app_name: str) -> Path:
@@ -99,6 +123,9 @@ class ImageManager(QObject):
             desktop_icon_path(app_name),
         )
 
+    def has_pixmaps(self, app_name: str) -> bool:
+        return all(file.is_file() for file in self.__img_all(app_name))
+
     def __prepare_download(self, game: Game, force: bool = False) -> Tuple[List, Dict]:
         if force and image_dir_game(game.app_name).exists():
             for file in self.__img_all(game.app_name):
@@ -109,6 +136,7 @@ class ImageManager(QObject):
         # Load image checksums
         if not self.__img_json(game.app_name).is_file():
             json_data: Dict = dict(zip(self.__img_types, [None] * len(self.__img_types)))
+            json_data["version"] = self.__cache_version
         else:
             json_data = json.load(open(self.__img_json(game.app_name), "r"))
 
@@ -121,7 +149,7 @@ class ImageManager(QObject):
             )
             try:
                 best = matches[0]
-            except IndexError as e:
+            except IndexError:
                 best = {}
             return best
 
@@ -140,20 +168,23 @@ class ImageManager(QObject):
         # lk: so everything below it is skipped
         # TODO: Move this into the thread, maybe, concurrency could help here too
         updates = []
-        if not all(file.is_file() for file in self.__img_all(game.app_name)):
-            # lk: fast path for games without images, convert Rare's logo
+        if not self.has_pixmaps(game.app_name):
             if not candidates:
+                # lk: fast path for games without images, convert Rare's logo
                 cache_data: Dict = dict(zip(self.__img_types, [None] * len(self.__img_types)))
                 with open(resources_path.joinpath("images", "cover.png"), "rb") as fd:
                     cache_data["DieselGameBoxTall"] = fd.read()
-                    fd.seek(0)
+                with open(resources_path.joinpath("images", "cover.png"), "rb") as fd:
                     cache_data["DieselGameBoxWide"] = fd.read()
                 # cache_data["DieselGameBoxLogo"] = open(
                 #         resources_path.joinpath("images", "logo.png"), "rb").read()
                 self.__convert(game, cache_data)
                 json_data["cache"] = None
                 json_data["scale"] = ImageSize.Tall.pixel_ratio
-                json_data["size"] = {"w": ImageSize.Tall.size.width(), "h": ImageSize.Tall.size.height()}
+                json_data["size"] = {
+                    "w": ImageSize.Tall.size.width(),
+                    "h": ImageSize.Tall.size.height(),
+                }
                 with open(self.__img_json(game.app_name), "w", encoding="utf-8") as file:
                     json.dump(json_data, file)
             else:
@@ -166,10 +197,10 @@ class ImageManager(QObject):
 
         return updates, json_data
 
-    def __download(self, updates: List, json_data: Dict, game: Game, use_async: bool = False) -> bool:
+    def __download(self, updates: List, json_data: Dict, game: Game) -> bool:
         # Decompress existing image.cache
         if not self.__img_cache(game.app_name).is_file():
-            cache_data = dict(zip(self.__img_types, [None] * len(self.__img_types)))
+            cache_data: Dict[str, Any] = dict(zip(self.__img_types, [None] * len(self.__img_types)))
         else:
             cache_data = self.__decompress(game)
 
@@ -181,45 +212,55 @@ class ImageManager(QObject):
             if (cache_data.get(image["type"], None) is None or json_data[image["type"]] != image["md5"])
         ]
 
-        # Download
-        # # lk: Keep this here, so I don't have to go looking for it again,
-        # # lk: it might be useful in the future.
-        # if use_async:
-        #     session = FuturesSession(max_workers=len(self.__img_types))
-        #     image_requests = []
-        #     for image in downloads:
-        #         logger.info(f"Downloading {image['type']} for {game.app_title}")
-        #         json_data[image["type"]] = image["md5"]
-        #         if image["type"] in self.__img_tall_types:
-        #             payload = {"resize": 1, "w": ImageSize.Image.size.width(), "h": ImageSize.Image.size.height()}
-        #         elif image["type"] in self.__img_wide_types:
-        #             payload = {"resize": 1, "w": ImageSize.ImageWide.size.width(), "h": ImageSize.ImageWide.size.height()}
-        #         else:
-        #             # Set the larger of the sizes for everything else
-        #             payload = {"resize": 1, "w": ImageSize.ImageWide.size.width(), "h": ImageSize.ImageWide.size.height()}
-        #         req = session.get(image["url"], params=payload)
-        #         req.image_type = image["type"]
-        #         image_requests.append(req)
-        #     for req in futures.as_completed(image_requests):
-        #         cache_data[req.image_type] = req.result().content
-        # else:
         for image in downloads:
-            logger.info(f"Downloading {image['type']} for {game.app_name} ({game.app_title})")
+            self.logger.debug(
+                "Downloading %s for %s (%s)",
+                image["type"],
+                game.app_name,
+                game.app_title,
+            )
             json_data[image["type"]] = image["md5"]
             if image["type"] in self.__img_tall_types:
-                payload = {"resize": 1, "w": ImageSize.Tall.size.width(), "h": ImageSize.Tall.size.height()}
+                payload = {
+                    "resize": 1,
+                    "w": ImageSize.Tall.size.width(),
+                    "h": ImageSize.Tall.size.height(),
+                }
             elif image["type"] in self.__img_wide_types:
-                payload = {"resize": 1, "w": ImageSize.Wide.size.width(), "h": ImageSize.Wide.size.height()}
+                payload = {
+                    "resize": 1,
+                    "w": ImageSize.Wide.size.width(),
+                    "h": ImageSize.Wide.size.height(),
+                }
             else:
                 # Set the larger of the sizes for everything else
-                payload = {"resize": 1, "w": ImageSize.Wide.size.width(), "h": ImageSize.Wide.size.height()}
+                payload = {
+                    "resize": 1,
+                    "w": ImageSize.Wide.size.width(),
+                    "h": ImageSize.Wide.size.height(),
+                }
             try:
-                # cache_data[image["type"]] = requests.get(image["url"], params=payload).content
                 cache_data[image["type"]] = requests.get(image["url"], params=payload, timeout=10).content
             except Exception as e:
-                logger.error(e)
+                self.logger.error(e)
                 return False
 
+        # lk: test the cached and downloaded data if they describe an image with valid dimensions
+        # I do not like this, it should add a bunch of processing for something simple but I am out of ideas
+        for image in updates:
+            image_data = QImage().fromData(cache_data[image["type"]])
+            if not (image_data.width() and image_data.height()):
+                with open(resources_path.joinpath("images", "cover.png"), "rb") as fd:
+                    cache_data[image["type"]] = fd.read()
+                json_data[image["type"]] = None
+                self.logger.error(
+                    "Invalid image %s data for %s (%s)",
+                    image["type"],
+                    game.app_name,
+                    game.app_title,
+                )
+            del image_data
+
         self.__convert(game, cache_data)
         # lk: don't keep the cache if there is no logo (kept for me)
         # if cache_data["DieselGameBoxLogo"] is not None:
@@ -235,7 +276,10 @@ class ImageManager(QObject):
 
         json_data["cache"] = archive_hash
         json_data["scale"] = ImageSize.Tall.pixel_ratio
-        json_data["size"] = {"w": ImageSize.Tall.size.width(), "h": ImageSize.Tall.size.height()}
+        json_data["size"] = {
+            "w": ImageSize.Tall.size.width(),
+            "h": ImageSize.Tall.size.height(),
+        }
 
         # write image.json
         with open(self.__img_json(game.app_name), "w", encoding="utf-8") as file:
@@ -245,10 +289,9 @@ class ImageManager(QObject):
 
     __icon_overlay: Optional[QPainterPath] = None
 
-    @staticmethod
-    def __generate_icon_overlay(rect: QRect) -> QPainterPath:
-        if ImageManager.__icon_overlay is not None:
-            return ImageManager.__icon_overlay
+    def __generate_icon_overlay(self, rect: QRect) -> QPainterPath:
+        if self.__icon_overlay is not None:
+            return self.__icon_overlay
         rounded_path = QPainterPath()
         margin = 0.05
         rounded_path.addRoundedRect(
@@ -261,11 +304,50 @@ class ImageManager(QObject):
             rect.height() * 0.2,
             rect.height() * 0.2,
         )
-        ImageManager.__icon_overlay = rounded_path
-        return ImageManager.__icon_overlay
+        self.__icon_overlay = rounded_path
+        return self.__icon_overlay
 
-    @staticmethod
-    def __convert_icon(cover: QImage) -> QImage:
+    def __convert_image(self, image_data, logo_data, preset: ImageSize.Preset) -> QImage:
+        image = QImage()
+        image.loadFromData(image_data)
+        image.convertToFormat(QImage.Format.Format_ARGB32_Premultiplied)
+        if image.height() > image.width() and preset.orientation == ImageType.Wide:
+            image = image.transformed(QTransform().rotate(90.0))
+        if image.height() < image.width() and preset.orientation == ImageType.Tall:
+            image = image.transformed(QTransform().rotate(-90.0))
+        # lk: Images are not always at the correct aspect ratio, so crop them to size
+        wr, hr = preset.aspect_ratio
+        factor = min(image.width() // wr, image.height() // hr)
+        rem_w = (image.width() - factor * wr) // 2
+        rem_h = (image.height() - factor * hr) // 2
+        image = image.copy(rem_w, rem_h, factor * wr, factor * hr)
+
+        if logo_data is not None:
+            logo = QImage()
+            logo.loadFromData(logo_data)
+            logo.convertToFormat(QImage.Format.Format_ARGB32_Premultiplied)
+            if logo.width() > image.width():
+                logo = logo.scaled(
+                    image.width(),
+                    image.height(),
+                    Qt.AspectRatioMode.KeepAspectRatio,
+                    Qt.TransformationMode.SmoothTransformation,
+                )
+            painter = QPainter(image)
+            painter.drawImage(
+                (image.width() - logo.width()) // 2,
+                image.height() - logo.height(),
+                logo,
+            )
+            painter.end()
+
+        return image.scaled(
+            preset.size,
+            Qt.AspectRatioMode.KeepAspectRatio,
+            Qt.TransformationMode.SmoothTransformation,
+        )
+
+    def __convert_icon(self, cover: QImage) -> QImage:
         icon_size = QSize(128, 128)
         icon = QImage(icon_size, QImage.Format.Format_ARGB32_Premultiplied)
         painter = QPainter(icon)
@@ -273,7 +355,7 @@ class ImageManager(QObject):
         painter.setRenderHint(QPainter.RenderHint.Antialiasing, True)
         painter.setCompositionMode(QPainter.CompositionMode.CompositionMode_Source)
         painter.fillRect(icon.rect(), Qt.GlobalColor.transparent)
-        overlay = ImageManager.__generate_icon_overlay(icon.rect())
+        overlay = self.__generate_icon_overlay(icon.rect())
         brush = QBrush(cover)
         scale = max(icon.width() / cover.width(), icon.height() / cover.height())
         transform = QTransform().scale(scale, scale)
@@ -285,6 +367,20 @@ class ImageManager(QObject):
         painter.end()
         return icon
 
+    def __save_image(self, image: QImage, color_path: Path, gray_path: Path):
+        # this is not required if we ever want to re-apply the alpha channel
+        # image = image.convertToFormat(QImage.Format_Indexed8)
+        # add the alpha channel back to the cover
+        image = image.convertToFormat(QImage.Format.Format_ARGB32_Premultiplied)
+        image.save(color_path.as_posix(), format="PNG")
+        # quick way to convert to grayscale, but keep the alpha channel
+        alpha = image.convertToFormat(QImage.Format.Format_Alpha8)
+        image = image.convertToFormat(QImage.Format.Format_Grayscale8)
+        # add the alpha channel back to the grayscale cover
+        image = image.convertToFormat(QImage.Format.Format_ARGB32_Premultiplied)
+        image.setAlphaChannel(alpha)
+        image.save(gray_path.as_posix(), format="PNG")
+
     def __convert(self, game, images, force=False) -> None:
         for file in self.__img_all(game.app_name):
             if force and file.exists():
@@ -302,61 +398,33 @@ class ImageManager(QObject):
         wide_data = find_image_data(self.__img_wide_types)
         logo_data = find_image_data(self.__img_logo_types)
 
-        def convert_image(image_data, logo_data, preset: ImageSize.Preset) -> QImage:
-            image = QImage()
-            image.loadFromData(image_data)
-            image.convertToFormat(QImage.Format.Format_ARGB32_Premultiplied)
-            # lk: Images are not always at the correct aspect ratio, so crop them to size
-            wr, hr = preset.aspect_ratio
-            factor = min(image.width() // wr, image.height() // hr)
-            rem_w = (image.width() - factor * wr) // 2
-            rem_h = (image.height() - factor * hr) // 2
-            image = image.copy(rem_w, rem_h, factor * wr, factor * hr)
+        tall = self.__convert_image(tall_data, logo_data, ImageSize.Tall)
+        self.__save_image(
+            tall,
+            image_tall_path(game.app_name),
+            image_tall_path(game.app_name, color=False),
+        )
 
-            if logo_data is not None:
-                logo = QImage()
-                logo.loadFromData(logo_data)
-                logo.convertToFormat(QImage.Format.Format_ARGB32_Premultiplied)
-                if logo.width() > image.width():
-                    logo = logo.scaled(
-                        image.width(),
-                        image.height(),
-                        Qt.AspectRatioMode.KeepAspectRatio,
-                        Qt.TransformationMode.SmoothTransformation
-                    )
-                painter = QPainter(image)
-                painter.drawImage((image.width() - logo.width()) // 2, image.height() - logo.height(), logo)
-                painter.end()
+        if wide_data is None:
+            wide_data = tall_data
 
-            return image.scaled(
-                preset.size,
-                Qt.AspectRatioMode.KeepAspectRatio,
-                Qt.TransformationMode.SmoothTransformation
-            )
-
-        tall = convert_image(tall_data, logo_data, ImageSize.Tall)
-        wide = convert_image(wide_data, logo_data, ImageSize.Wide)
+        wide = self.__convert_image(wide_data, logo_data, ImageSize.Wide)
+        self.__save_image(
+            wide,
+            image_wide_path(game.app_name),
+            image_wide_path(game.app_name, color=False),
+        )
 
         icon = self.__convert_icon(tall)
-        icon.save(desktop_icon_path(game.app_name).as_posix(), format=desktop_icon_suffix().upper())
-
-        def save_image(image: QImage, color_path: Path, gray_path: Path):
-            # this is not required if we ever want to re-apply the alpha channel
-            # image = image.convertToFormat(QImage.Format_Indexed8)
-            # add the alpha channel back to the cover
-            image = image.convertToFormat(QImage.Format.Format_ARGB32_Premultiplied)
-            image.save(color_path.as_posix(), format="PNG")
-            # quick way to convert to grayscale, but keep the alpha channel
-            alpha = image.convertToFormat(QImage.Format.Format_Alpha8)
-            image = image.convertToFormat(QImage.Format.Format_Grayscale8)
-            # add the alpha channel back to the grayscale cover
-            image = image.convertToFormat(QImage.Format.Format_ARGB32_Premultiplied)
-            image.setAlphaChannel(alpha)
-            image.save(gray_path.as_posix(), format="PNG")
-
-        save_image(icon, image_icon_path(game.app_name), image_icon_path(game.app_name, color=False))
-        save_image(tall, image_tall_path(game.app_name), image_tall_path(game.app_name, color=False))
-        save_image(wide, image_wide_path(game.app_name), image_wide_path(game.app_name, color=False))
+        self.__save_image(
+            icon,
+            image_icon_path(game.app_name),
+            image_icon_path(game.app_name, color=False),
+        )
+        icon.save(
+            desktop_icon_path(game.app_name).as_posix(),
+            format=desktop_icon_suffix().upper(),
+        )
 
     def __compress(self, game: Game, data: Dict) -> None:
         archive = open(self.__img_cache(game.app_name), "wb")
@@ -375,34 +443,38 @@ class ImageManager(QObject):
             archive.close()
         return data
 
-    def __append_to_queue(self, game: Game):
-        self.__worker_app_names.add(game.app_name)
+    def __append_to_worker_queue(self, game: Game):
+        self.__worker_lock.acquire()
+        try:
+            self.__worker_app_names.add(game.app_name)
+        finally:
+            self.__worker_lock.release()
 
     @Slot(object)
-    def __remove_from_queue(self, game: Game):
-        self.__worker_app_names.remove(game.app_name)
+    def __remove_from_worker_queue(self, game: Game):
+        self.__worker_lock.acquire()
+        try:
+            self.__worker_app_names.remove(game.app_name)
+        finally:
+            self.__worker_lock.release()
 
-    def download_image(
-            self, game: Game, load_callback: Callable[[], None], priority: int, force: bool = False
-    ) -> None:
+    def __download_image(self, game, force: bool):
+        updates, json_data = self.__prepare_download(game, force)
+        if updates:
+            self.__download(updates, json_data, game)
+        self.logger.debug("Emitting singal for %s (%s})", game.app_name, game.app_title)
+
+    def download_image(self, game: Game, load_callback: Callable[[], None], priority: int, force: bool = False) -> None:
         if game.app_name in self.__worker_app_names:
             return
-        self.__append_to_queue(game)
-        updates, json_data = self.__prepare_download(game, force)
-        if not updates:
-            self.__remove_from_queue(game)
-            load_callback()
-        else:
-            # Copy the data because we are going to be a thread and we modify them later on
-            image_worker = ImageManager.Worker(self.__download, updates.copy(), json_data.copy(), game)
-            image_worker.signals.completed.connect(self.__remove_from_queue)
-            image_worker.signals.completed.connect(load_callback)
-            self.threadpool.start(image_worker, priority)
+        self.__append_to_worker_queue(game)
+        image_worker = ImageWorker(self.__download_image, game, force)
+        image_worker.signals.completed.connect(load_callback)
+        image_worker.signals.completed.connect(self.__remove_from_worker_queue)
+        self.threadpool.start(image_worker, priority)
 
-    def download_image_launch(
-            self, game: Game, callback: Callable[[Game], None], priority: int, force: bool = False
-    ) -> None:
-        if self.__img_cache(game.app_name).is_file() and not force:
+    def download_image_launch(self, game: Game, callback: Callable[[Game], None], priority: int, force: bool = False) -> None:
+        if self.has_pixmaps(game.app_name) and not force:
             return
 
         def _callback():
@@ -410,16 +482,18 @@ class ImageManager(QObject):
 
         self.download_image(game, _callback, priority, force)
 
-    def download_image_blocking(self, game: Game, force: bool = False) -> None:
-        updates, json_data = self.__prepare_download(game, force)
-        if not updates:
-            return
-        if updates:
-            self.__download(updates, json_data, game, use_async=True)
+    def download_image_blocking(self, game: Game, load_callback: Callable[[], None], priority: int, force: bool = False) -> None:
+        self.__append_to_worker_queue(game)
+        self.__download_image(game, force)
+        load_callback()
+        self.__remove_from_worker_queue(game)
 
     @staticmethod
     def __get_cover(
-            container: Union[Type[QPixmap], Type[QImage]], app_name: str, preset: ImageSize.Preset, color: bool,
+        container: Union[Type[QPixmap], Type[QImage]],
+        app_name: str,
+        preset: ImageSize.Preset,
+        color: bool,
     ) -> Union[QPixmap, QImage]:
         ret = container()
         if preset.orientation == ImageType.Icon:
@@ -436,17 +510,17 @@ class ImageManager(QObject):
         if not ret.isNull():
             device = ImageSize.Preset(
                 divisor=preset.base.divisor,
-                pixel_ratio=QApplication.instance().devicePixelRatio(),
+                pixel_ratio=1,
                 orientation=preset.base.orientation,
-                base=preset
+                base=preset,
             )
             ret.setDevicePixelRatio(preset.pixel_ratio)
             # lk: Scaling happens at painting. It might be inefficient so leave this here as an alternative
-            # lk: If this is uncommented, the transformation in ImageWidget should be adjusted also
+            # lk: If this is uncommented, the transformation in ImageWidget should be adjusted too
             ret = ret.scaled(
                 device.size,
                 Qt.AspectRatioMode.KeepAspectRatio,
-                Qt.TransformationMode.SmoothTransformation
+                Qt.TransformationMode.SmoothTransformation,
             )
             ret.setDevicePixelRatio(device.pixel_ratio)
         return ret
