@@ -4,67 +4,67 @@ import time
 from argparse import Namespace
 from itertools import chain
 from logging import getLogger
-from typing import Dict, Iterator, Callable, Optional, List, Union, Iterable, Tuple, Set
+from typing import Callable, Dict, Iterable, Iterator, List, Optional, Set, Tuple, Union
 
-from PySide6.QtCore import QObject, Signal, QSettings, Slot, QThreadPool, QRunnable, QTimer
 from legendary.lfs.eos import EOSOverlayApp
 from legendary.models.game import Game, SaveGameFile
-from requests.exceptions import HTTPError, ConnectionError
+from PySide6.QtCore import QObject, QRunnable, QThreadPool, QTimer, Signal, Slot
+from requests.exceptions import ConnectionError, HTTPError
 
 from rare.lgndr.core import LegendaryCore
 from rare.models.base_game import RareSaveGame
-from rare.models.game import RareGame, RareEosOverlay
+from rare.models.game import RareEosOverlay, RareGame
+from rare.models.settings import RareAppSettings
 from rare.models.signals import GlobalSignals
+from rare.utils import config_helper, steam_shortcuts
 from rare.utils.metrics import timelogger
-from rare.utils import config_helper
-from rare.utils.steam_shortcuts import load_steam_shortcuts
+
 from .image_manager import ImageManager
 from .workers import (
-    QueueWorker,
-    VerifyWorker,
-    MoveWorker,
+    EntitlementsWorker,
     FetchWorker,
     GamesDlcsWorker,
-    EntitlementsWorker,
+    MoveWorker,
     OriginWineWorker,
+    QueueWorker,
+    VerifyWorker,
 )
+from .workers.fetch import SteamAppIdsWorker
 from .workers.uninstall import uninstall_game
 from .workers.worker import QueueWorkerInfo, QueueWorkerState
 from .wrappers import Wrappers
 
-logger = getLogger("RareCore")
-
 
 class RareCore(QObject):
     progress = Signal(int, str)
     completed = Signal()
-    # lk: these are unused but remain if case they are become relevant
+    # lk: these are unused but remain if case they become relevant
     # completed_saves = Signal()
     # completed_origin = Signal()
     # completed_entitlements = Signal()
 
     # lk: special case class attribute, this has to be here
-    __instance: Optional['RareCore'] = None
+    __instance: Optional["RareCore"] = None
 
-    def __init__(self, args: Namespace):
+    def __init__(self, settings: RareAppSettings, args: Namespace):
         if self.__instance is not None:
             raise RuntimeError("RareCore already initialized")
         super(RareCore, self).__init__()
+        self.logger = getLogger(type(self).__name__)
+        self.__settings = settings
         self.__args: Optional[Namespace] = None
         self.__signals: Optional[GlobalSignals] = None
         self.__core: Optional[LegendaryCore] = None
         self.__image_manager: Optional[ImageManager] = None
-        self.__settings: Optional[QSettings] = None
         self.__wrappers: Optional[Wrappers] = None
 
         self.__start_time = time.perf_counter()
 
-        self.args(args)
+        self.args(args=args)
         self.signals(init=True)
         self.core(init=True)
         config_helper.init_config_handler(self.__core)
         self.image_manager(init=True)
-        self.__settings = QSettings(self)
         self.__wrappers = Wrappers()
 
         self.queue_workers: List[QueueWorker] = []
@@ -72,13 +72,14 @@ class RareCore(QObject):
         self.queue_threadpool.setMaxThreadCount(2)
 
         self.__library: Dict[str, RareGame] = {}
-        self.__eos_overlay = RareEosOverlay(self.__core, EOSOverlayApp, self)
+        self.__eos_overlay = RareEosOverlay(self.__core, EOSOverlayApp)
         self.__eos_overlay.signals.game.install.connect(self.__signals.game.install)
         self.__eos_overlay.signals.game.uninstall.connect(self.__signals.game.uninstall)
 
         self.__fetch_progress: int = 0
         self.__fetched_games_dlcs: bool = False
         self.__fetched_entitlements: bool = False
+        self.__fetched_steamappids: bool = False
 
         RareCore.__instance = self
 
@@ -116,7 +117,8 @@ class RareCore(QObject):
             yield w.worker_info()
 
     @staticmethod
-    def instance() -> 'RareCore':
+    def instance() -> "RareCore":
+        raise RuntimeError("RareCore.instance() method is deprecated")
         if RareCore.__instance is None:
             raise RuntimeError("Uninitialized use of RareCore")
         return RareCore.__instance
@@ -148,18 +150,20 @@ class RareCore(QObject):
             try:
                 self.__core = LegendaryCore()
             except configparser.MissingSectionHeaderError as e:
-                logger.warning("Config is corrupt: %s", e)
-                if config_path := os.environ.get('LEGENDARY_CONFIG_PATH'):
+                self.logger.warning("Config is corrupt: %s", e)
+                if config_path := os.environ.get("LEGENDARY_CONFIG_PATH"):
                     path = config_path
-                elif config_path := os.environ.get('XDG_CONFIG_HOME'):
-                    path = os.path.join(config_path, 'legendary')
+                elif config_path := os.environ.get("XDG_CONFIG_HOME"):
+                    path = os.path.join(config_path, "legendary")
                 else:
-                    path = os.path.expanduser('~/.config/legendary')
-                logger.info("Creating config in path: %s", config_path)
+                    path = os.path.expanduser("~/.config/legendary")
+                self.logger.info("Creating config in path: %s", config_path)
                 with open(os.path.join(path, "config.ini"), "w", encoding="utf-8") as config_file:
                     config_file.write("[Legendary]")
                 self.__core = LegendaryCore()
 
+            self.__core.egs._store_gql_host = "launcher.store.epicgames.com"
+
             # Initialize sections if they don't exist
             for section in ["Legendary", "default", "default.env"]:
                 if section not in self.__core.lgd.config.sections():
@@ -175,12 +179,12 @@ class RareCore(QObject):
             if not check_config("default_platform", {"Windows", "Win32", "Mac"}):
                 self.__core.lgd.config.set("Legendary", "default_platform", self.__core.default_platform)
             if not check_config("install_dir"):
-                self.__core.lgd.config.set(
-                    "Legendary", "install_dir", self.__core.get_default_install_dir()
-                )
+                self.__core.lgd.config.set("Legendary", "install_dir", self.__core.get_default_install_dir())
             if not check_config("mac_install_dir"):
                 self.__core.lgd.config.set(
-                    "Legendary", "mac_install_dir", self.__core.get_default_install_dir(self.__core.default_platform)
+                    "Legendary",
+                    "mac_install_dir",
+                    self.__core.get_default_install_dir(self.__core.default_platform),
                 )
 
             # Always set these options
@@ -215,9 +219,6 @@ class RareCore(QObject):
     def wrappers(self) -> Wrappers:
         return self.__wrappers
 
-    def settings(self) -> QSettings:
-        return self.__settings
-
     def deleteLater(self) -> None:
         self.__image_manager.deleteLater()
         del self.__image_manager
@@ -237,6 +238,8 @@ class RareCore(QObject):
         # del self.__eos_overlay
         self.__eos_overlay = None
 
+        for rgame in self.__instance.games_and_dlcs:
+            rgame.deleteLater()
         RareCore.__instance = None
         super(RareCore, self).deleteLater()
 
@@ -246,14 +249,12 @@ class RareCore(QObject):
             # lk: to avoid spamming the log with "file not found" errors
             for dlc in rgame.owned_dlcs:
                 if dlc.is_installed:
-                    logger.info(f'Uninstalling DLC "{dlc.app_name}" ({dlc.app_title})...')
-                    uninstall_game(self.__core, dlc, keep_files=True, keep_config=True)
+                    self.logger.info(f'Uninstalling DLC "{dlc.app_name}" ({dlc.app_title})...')
+                    uninstall_game(self.__core, dlc, self.logger, keep_files=True, keep_config=True)
                     dlc.igame = None
-            logger.info(
-                f'Removing "{rgame.app_title}" because "{rgame.igame.install_path}" does not exist...'
-            )
-            uninstall_game(self.__core, rgame, keep_files=True, keep_config=True)
-            logger.info(f"Uninstalled {rgame.app_title}, because no game files exist")
+            self.logger.info(f'Removing "{rgame.app_title}" because "{rgame.igame.install_path}" does not exist...')
+            uninstall_game(self.__core, rgame, self.logger, keep_files=True, keep_config=True)
+            self.logger.info(f"Uninstalled {rgame.app_title}, because no game files exist")
             rgame.igame = None
             return
         # lk: games that don't have an override and can't find their executable due to case sensitivity
@@ -266,11 +267,11 @@ class RareCore(QObject):
         # lk: Case-insensitive search for the game's executable (example: Brothers - A Tale of two Sons)
         executable_path = os.path.join(rgame.igame.install_path, igame_executable.replace("\\", "/").lstrip("/"))
         file_list = map(str.lower, os.listdir(os.path.dirname(executable_path)))
-        if not os.path.basename(executable_path).lower() in file_list:
+        if os.path.basename(executable_path).lower() not in file_list:
             rgame.igame.needs_verification = True
             self.__core.lgd.set_installed_game(rgame.app_name, rgame.igame)
             rgame.update_igame()
-            logger.info(f"{rgame.app_title} needs verification")
+            self.logger.info(f"{rgame.app_title} needs verification")
 
     def get_game(self, app_name: str) -> Union[RareEosOverlay, RareGame]:
         if app_name == EOSOverlayApp.app_name:
@@ -303,11 +304,11 @@ class RareCore(QObject):
 
     def __create_or_update_rgame(self, game: Game) -> RareGame:
         if rgame := self.__library.get(game.app_name, False):
-            logger.warning(f"{rgame.app_name} already present in {type(self).__name__}")
-            logger.info(f"Updating Game for {rgame.app_name}")
+            self.logger.warning(f"{rgame.app_name} already present in {type(self).__name__}")
+            self.logger.info(f"Updating Game for {rgame.app_name}")
             rgame.update_rgame()
         else:
-            rgame = RareGame(self.__core, self.__image_manager, game, self)
+            rgame = RareGame(self.__settings, self.__core, self.__image_manager, game)
             self.__add_game(rgame)
         return rgame
 
@@ -320,17 +321,17 @@ class RareCore(QObject):
                     rdlc = self.__create_or_update_rgame(dlc)
                     if rdlc not in rgame.owned_dlcs:
                         rgame.add_dlc(rdlc)
-            # lk: since loading has to know about game state,
-            # validate installation just adding each RareGamesu
+            # lk: since loading has to know about the game's state,
+            # validate installation just by trying to add each RareGame
             # TODO: this should probably be moved into RareGame
             if rgame.is_installed and not (rgame.is_dlc or rgame.is_non_asset):
                 try:
                     self.__validate_install(rgame)
                 except FileNotFoundError as e:
-                    logger.info(f'Marking "{rgame.app_title}" as not installed because an exception has occurred...')
-                    logger.error(e)
+                    self.logger.info(f'Marking "{rgame.app_title}" as not installed because an exception has occurred...')
+                    self.logger.error(e)
                     rgame.set_installed(False)
-            progress = int(idx/length * self.__fetch_progress) + (100 - self.__fetch_progress)
+            progress = int(idx / length * self.__fetch_progress) + (100 - self.__fetch_progress)
             self.progress.emit(progress, self.tr("Loaded <b>{}</b>").format(rgame.app_title))
 
     @Slot(int, str)
@@ -348,17 +349,27 @@ class RareCore(QObject):
             self.__core.lgd.entitlements = result
             self.__fetched_entitlements = True
 
-        logger.info("Acquired data from %s worker", FetchWorker.Result(result_type).name)
+        if result_type == FetchWorker.Result.STEAMAPPIDS:
+            self.__fetched_steamappids = True
+
+        self.logger.info("Acquired data from %s worker", FetchWorker.Result(result_type).name)
 
         # Return early if there are still things to fetch
-        if not all({self.__fetched_games_dlcs, self.__fetched_entitlements}):
+        if not all(
+            {
+                self.__fetched_games_dlcs,
+                self.__fetched_entitlements,
+                self.__fetched_steamappids,
+            }
+        ):
             return
 
-        logger.debug("Fetch time %s seconds", time.perf_counter() - self.__start_time)
-        self.__wrappers.import_wrappers(
-            self.__core, self.__settings, [rgame.app_name for rgame in self.games]
-        )
-        load_steam_shortcuts()
+        self.logger.debug("Fetch time %s seconds", time.perf_counter() - self.__start_time)
+        self.__wrappers.import_wrappers(self.__settings, self.__core, [rgame.app_name for rgame in self.games])
+
+        # Look for Rare shortcuts in Steam
+        steam_shortcuts.load_steam_shortcuts()
+
         self.progress.emit(100, self.tr("Launching Rare"))
         self.completed.emit()
         QTimer.singleShot(100, self.__post_init)
@@ -366,39 +377,44 @@ class RareCore(QObject):
     def fetch(self):
         self.__start_time = time.perf_counter()
 
-        games_dlcs_worker = GamesDlcsWorker(self.__core, self.__args)
+        games_dlcs_worker = GamesDlcsWorker(self.__settings, self.__core, self.__args)
         games_dlcs_worker.signals.progress.connect(self.__on_fetch_progress)
         games_dlcs_worker.signals.result.connect(self.__on_fetch_result)
 
-        entitlements_worker = EntitlementsWorker(self.__core, self.__args)
+        entitlements_worker = EntitlementsWorker(self.__settings, self.__core, self.__args)
         entitlements_worker.signals.progress.connect(self.__on_fetch_progress)
         entitlements_worker.signals.result.connect(self.__on_fetch_result)
 
+        steamappids_worker = SteamAppIdsWorker(self.__settings, self.__core, self.__args)
+        steamappids_worker.signals.progress.connect(self.__on_fetch_progress)
+        steamappids_worker.signals.result.connect(self.__on_fetch_result)
+
         QThreadPool.globalInstance().start(games_dlcs_worker)
         QThreadPool.globalInstance().start(entitlements_worker)
+        QThreadPool.globalInstance().start(steamappids_worker)
+
+    def __fetch_saves(self) -> None:
+        saves_dict: Dict[str, List[SaveGameFile]] = {}
+        try:
+            with timelogger(self.logger, "Request saves"):
+                saves_list = self.__core.get_save_games()
+            for s in saves_list:
+                if s.app_name not in saves_dict.keys():
+                    saves_dict[s.app_name] = [s]
+                else:
+                    saves_dict[s.app_name].append(s)
+            for app_name, saves in saves_dict.items():
+                if app_name not in self.__library.keys():
+                    continue
+                self.__library[app_name].load_saves(saves)
+        except (HTTPError, ConnectionError) as e:
+            self.logger.error("Exception while fetching saves from EGS.")
+            self.logger.error(e)
+            return
+        self.logger.info(f"Saves: {len(saves_dict)}")
 
     def fetch_saves(self):
-        def __fetch() -> None:
-            saves_dict: Dict[str, List[SaveGameFile]] = {}
-            try:
-                with timelogger(logger, "Request saves"):
-                    saves_list = self.__core.get_save_games()
-                for s in saves_list:
-                    if s.app_name not in saves_dict.keys():
-                        saves_dict[s.app_name] = [s]
-                    else:
-                        saves_dict[s.app_name].append(s)
-                for app_name, saves in saves_dict.items():
-                    if app_name not in self.__library.keys():
-                        continue
-                    self.__library[app_name].load_saves(saves)
-            except (HTTPError, ConnectionError) as e:
-                logger.error("Exception while fetching saves from EGS.")
-                logger.error(e)
-                return
-            logger.info(f"Saves: {len(saves_dict)}")
-
-        saves_worker = QRunnable.create(__fetch)
+        saves_worker = QRunnable.create(self.__fetch_saves)
         QThreadPool.globalInstance().start(saves_worker)
 
     def resolve_origin(self) -> None:
@@ -410,6 +426,15 @@ class RareCore(QObject):
             self.fetch_saves()
         self.resolve_origin()
 
+    @property
+    def game_tags(self) -> Tuple[str, ...]:
+        default_tags = ("favorite", "backlog", "completed", "hidden")
+        custom_tags = set()
+        for rgame in self.games:
+            custom_tags.update(rgame.tags)
+        custom_tags = custom_tags.difference(default_tags)
+        return *default_tags, *sorted(custom_tags)
+
     @property
     def games_and_dlcs(self) -> Iterator[RareGame]:
         for app_name in self.__library:
@@ -484,4 +509,3 @@ class RareCore(QObject):
         RareGames that have SaveGameFiles associated with them
         """
         return self.__filter_games(lambda game: bool(game.saves))
-
